/****************************************************************************
*  This file is part of the HYDRA application.
*
*  Copyright (c) 2004 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*
*  Name: nutsetup.c   
*
*  History:
*
*  22.10.04  mifi   First Version
****************************************************************************/
#define __NUTSETUP_C__

#include <stdio.h>
#include <string.h>
#include <io.h>

#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/heap.h>
#include <sys/confnet.h>
#include <sys/socket.h>
#include <sys/version.h>
#include <net/if_var.h>
#include <netinet/in.h>

#include "typedefs.h"
#include "nutsetup.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*
 * Define the Ethernut name here, not more than 16 chars.
 * This name will be used by the nutsetup
 */
#define NUT_SETUP_NAME      "nutsetup"

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static NUT_SETUP NutSetupSend;

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/
/***************************************************************************/
/*  SendResponse                                                           */
/*                                                                         */
/*  Send the response to the PC-Applikation.                               */
/*                                                                         */
/*  In    :                                                                */
/*  Out   :                                                                */
/*  Return:                                                                */
/***************************************************************************/
static void SendResponse (UDPSOCKET *Socket)
{
  BYTE bIndex;
  
  memset(&NutSetupSend, 0x00, sizeof(NUT_SETUP));
  
  NutSetupSend.wMagic = NUT_SETUP_MAGIC;
  NutSetupSend.wSize  = sizeof(NUT_SETUP);
  NutSetupSend.bMode  = NUT_SETUP_RESPONSE;
  
  for(bIndex=0; bIndex<6; bIndex++) {
    NutSetupSend.bMACAddress[bIndex] = confnet.cdn_mac[bIndex];
  }
    
  NutSetupSend.dAddress = confnet.cdn_ip_addr;
  NutSetupSend.dMask    = confnet.cdn_ip_mask;
  NutSetupSend.dGateway = confnet.cdn_gateway; 
  NutSetupSend.dVersion = NutVersion();
  
  if (strlen(NUT_SETUP_NAME) <= NUT_SETUP_NAME_LEN) {
    memcpy(NutSetupSend.Name, NUT_SETUP_NAME, strlen(NUT_SETUP_NAME));  
  }
  
  NutUdpSendTo(Socket, INADDR_BROADCAST, NUT_SETUP_UDP_PORT,  
               &NutSetupSend, sizeof(NUT_SETUP));
}

/***************************************************************************/
/*  HandleConfigurationPacket                                              */
/*                                                                         */
/*  Handle all the incoming packet here.                                   */
/*                                                                         */
/*  In    :                                                                */
/*  Out   :                                                                */
/*  Return:                                                                */
/***************************************************************************/
static void HandleConfigurationPacket (UDPSOCKET *Socket, NUT_SETUP *pNutSetup)
{
  switch (pNutSetup->bMode) {
    case NUT_SETUP_REQUEST: {
      SendResponse(Socket);
      break;
    }
    case NUT_SETUP_SET: {
      if ( (pNutSetup->bMACAddress[0] == confnet.cdn_mac[0]) &&
           (pNutSetup->bMACAddress[1] == confnet.cdn_mac[1]) &&
           (pNutSetup->bMACAddress[2] == confnet.cdn_mac[2]) &&
           (pNutSetup->bMACAddress[3] == confnet.cdn_mac[3]) &&
           (pNutSetup->bMACAddress[4] == confnet.cdn_mac[4]) &&
           (pNutSetup->bMACAddress[5] == confnet.cdn_mac[5]) ){
           
        confnet.cdn_ip_addr = pNutSetup->dAddress;
        confnet.cdn_ip_mask = pNutSetup->dMask;
        confnet.cdn_gateway = pNutSetup->dGateway;
        NutNetSaveConfig();
        NutSleep(500);
        asm("jmp 0x0000");  // Restart       
      }     
      break;
    }
  }  
}

/***************************************************************************/
/*  NutSetup                                                               */
/*                                                                         */
/*  This is our thread.                                                    */
/*                                                                         */
/*  In    :                                                                */
/*  Out   :                                                                */
/*  Return:                                                                */
/***************************************************************************/
THREAD(NutSetup, arg)
{
  int        nPacketLen;
  WORD       wFromPort;
  DWORD      dFromAddress;
  NUT_SETUP *pNutSetup;
  UDPSOCKET  *Socket;
  
  pNutSetup = NutHeapAlloc(sizeof(NUT_SETUP));  
  
  /*
   * Create a socket.
   */
  while ((Socket = NutUdpCreateSocket(NUT_SETUP_UDP_PORT)) == 0)
    NutSleep(5000);
    
  SendResponse(Socket);    
    
  for(;;) {
    nPacketLen = NutUdpReceiveFrom(Socket, &dFromAddress, &wFromPort, pNutSetup, sizeof(NUT_SETUP), 5000);
    if (nPacketLen > 0) {
      /*
       * Check for configuration packet
       */
      if( (nPacketLen >= sizeof(NUT_SETUP))        &&
          (pNutSetup->wMagic == NUT_SETUP_MAGIC)   &&
          (pNutSetup->wSize  == sizeof(NUT_SETUP)) ){
        HandleConfigurationPacket(Socket, pNutSetup);
      }
    }
  } /* end for */  
}

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  NutSetupStart                                                          */
/*                                                                         */
/*  In    :                                                                */
/*  Out   :                                                                */
/*  Return:                                                                */
/***************************************************************************/
void NutSetupStart (void)
{
  /*
   * Start out thread...
   */
  NutThreadCreate("SETUP", NutSetup, 0, 640);  
}